use rand::{distributions::Alphanumeric, Rng};

mod utils;
use utils::{server::ServerTest, client::ClientTest};


fn get_random_test_id() -> String {
    let mut id: String = rand::thread_rng()
        .sample_iter(&Alphanumeric)
        .take(7)
        .map(char::from)
        .collect();
    id.push_str("_nextsync");
    id.to_owned()
}

#[cfg(test)]
mod push_tests {
    use super::*;

    #[test]
    fn simple_push() {
        let id = get_random_test_id();
        let mut server = ServerTest::new(id.clone()).init();
        let mut client = ClientTest::new(id).init();

        let _ = client.add_file("file1", "foo");
        client.run_cmd_ok("add file1");
        client.run_cmd_ok("push");

        // tests
        assert!(server.has_file("file1", "foo"));

        client.clean();
        server.clean();
    }

    #[test]
    fn push_update() {
        let id = get_random_test_id();
        let mut server = ServerTest::new(id.clone()).init();
        let mut client = ClientTest::new(id).init();

        // init content of file1
        let _ = client.add_file("file1", "foo");
        client.run_cmd_ok("add file1");
        client.run_cmd_ok("push");

        // tests
        assert!(server.has_file("file1", "foo"));

        // change content of file1
        let _ = client.add_file("file1", "bar");
        client.run_cmd_ok("add file1");
        client.run_cmd_ok("push");

        // tests
        assert!(server.has_file("file1", "bar"));

        client.clean();
        server.clean();
    }

    #[test]
    fn push_dir() {
        let id = get_random_test_id();
        let mut server = ServerTest::new(id.clone()).init();
        let mut client = ClientTest::new(id).init();

        let _ = client.add_dir("dir");
        let _ = client.add_file("dir/file2", "bar");

        // push dir and file2
        client.run_cmd_ok("add file2");
        client.run_cmd_ok("push");

        // tests
        assert!(server.has_file("dir/file2", "bar"));

        client.clean();
        server.clean();
    }

    #[test]
    fn push_all() {
        let id = get_random_test_id();
        let mut server = ServerTest::new(id.clone()).init();
        let mut client = ClientTest::new(id).init();

        let _ = client.add_file("file1", "foo");
        let _ = client.add_dir("dir");
        let _ = client.add_file("dir/file2", "bar");

        // push dir and file2
        client.run_cmd_ok("add *");
        client.run_cmd_ok("push");

        // tests
        assert!(server.has_file("file1", "foo"));
        assert!(server.has_file("dir/file2", "bar"));

        client.clean();
        server.clean();
    }
}
